package cn.com.qmth.stmms.ms.admin.api;

import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.user.MarkRight;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.MarkSubjectRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkTaskRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkUserRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author ting.yin
 * @Description: 用户api
 * @date 2016年10月24日
 */
@RestController
@RequestMapping("api/admin/users")
public class UserApi {

    @Autowired
    private MarkUserRepo markUserRepo;

    @Autowired
    private MarkSubjectRepo markSubjectRepo;

    @Autowired
    private MarkTaskRepo markTaskRepo;

    @RequestMapping(value = "{userId}", method = RequestMethod.DELETE)
    public void remove(@PathVariable Long userId) {
        long count = markTaskRepo.countByMarkerId(userId);
        if (count > 0) {
            throw new RuntimeException("已分配任务，不允许删除");
        }
        markUserRepo.delete(userId);
    }

    @RequestMapping(value = "{domain}", method = RequestMethod.PUT)
    public void update(@PathVariable MarkUser domain, @RequestBody MarkUser user) {
        MarkSubject markSubject = markSubjectRepo.findOne(user.getWorkId() + "-" + user.getSubject().toString());
        if (domain.getRole().equals(Role.MARKER)) {
            if (user.getWeight() - domain.getWeight() != 0 && !markSubject.getStage().equals(MarkStage.INIT)) {
                throw new RuntimeException("请在分档阶段前修改权重");
            }
            if (!(user.getWeight() >= 1)) {
                throw new RuntimeException("评卷员权重必须大于1");
            }
        }
        domain.setName(user.getName());
        domain.setMarkRight(user.getMarkRight());
        domain.setWeight(user.getWeight());
        markUserRepo.save(domain);
    }

    @RequestMapping(method = RequestMethod.POST)
    public void create(@RequestBody MarkUser user) {
        MarkSubject markSubject = markSubjectRepo.findOne(user.getWorkId() + "-" + user.getSubject().toString());
        if (markSubject == null) {
            throw new RuntimeException("该科目不存在");
        }
        if (user.getRole() == Role.MARKER && user.getMarkRight() == MarkRight.ALLOW_LEVELING && !markSubject.getStage().equals(MarkStage.INIT)) {
            throw new RuntimeException("请进入分档阶段前创建");
        }
        MarkUser markUser = markUserRepo.findByLoginName(user.getLoginName());
        if (markUser != null) {
            throw new RuntimeException("该登录名已经存在");
        }
        user.setLoginName(user.getLoginName());
        if (Objects.equals(user.getRole().name(), Role.MARKER.name())) {
            user.setOneClickLevel(false);
        }
        markUserRepo.save(user);
    }

    @RequestMapping(method = RequestMethod.GET)
    public List<MarkUser> list(@RequestParam Long workId,
                               @RequestParam Subject subject,
                               @RequestParam(required = false) Role role) {
        if (role != null) {
            return markUserRepo.findByWorkIdAndRole(workId, role).stream().filter(u -> u.getSubject() == subject)
                    .collect(Collectors.toList());
        } else {
            return markUserRepo.findByWorkId(workId).stream().filter(u -> u.getSubject() == subject)
                    .collect(Collectors.toList());
        }
    }


}
